/**
	Universidad de la Repblica - Facultad de Ingeniera
	Instituto de Computacin - Instituto de Ingeniera Elctrica
	Proyecto AntelSat

	@file		ukernel.h
	@version	Vuelo
	@date		03/05/2014
	@author		Gustavo De Martino

	@brief		Declaraciones relativas a la versin 0.5 del micro kernel para MSP430
*/

#ifndef __UKERNEL_H__
#define __UKERNEL_H__

#include <stddef.h>
#include <inttypes.h>

#include "ukernel_config.h"

/* Espera indefinida hasta que suceda el evento */
#define TIMEOUT_NO_TIMEOUT 0

#define NO_PID 0xFFFF
#define NO_RESOURCE 0xFFFF
#define RESOURCE_IDLE 0

/* Resultados posibles de las operaciones */
typedef enum {
	ERROR_NO_ERROR,
	ERROR_NO_PROGS,
	ERROR_TOO_MANY_PROGS,
	ERROR_TIMEOUT,
	ERROR_NOT_LOCKED,
	ERROR_ALREADY_LOCKED,
	ERROR_DATA_LOST,
	ERROR_BUFFER_NOT_READY,
	ERROR_BUFFER_FULL,
	ERROR_INVALID_CRC,
	ERROR_INVALID_DATA_SIZE,
	ERROR_INVALID_MODULE,
	ERROR_INVALID_APPLICATION,
	ERROR_INVALID_RESOURCE,
	ERROR_INVALID_OWNER,
	ERROR_INVALID_BUFFER,
	ERROR_INVALID_MESSAGE,
	ERROR_INVALID_STATUS,
	ERROR_TX_DISABLED,
	ERROR_TX_ERROR
} error_t;

/* Estado de un recurso */
typedef enum {
	RESOURCE_STATUS_FREE		= 0x0000,
	RESOURCE_STATUS_LOCKED		= 0x0007,
	RESOURCE_STATUS_DATA_WAIT	= 0x0038,
	RESOURCE_STATUS_DATA_READY	= 0x01C0,
	RESOURCE_STATUS_INVALID		= 0xFFFF
} resource_status_t;

typedef unsigned short resource_t;

typedef unsigned short stack_t;

/**
 *	kernel_init
 *	La funcin kernel_init controla la integridad del cdigo e inicializa los datos del kernel.
 *	No inicializa el hardware y debe ejecutarse antes de cargar los programas.
 *
 *	@return	Cdigo de error
 */
error_t kernel_init( void );

/**
 *	load
 *	@brief Prepara un proceso para ser ejecutado inicializando su contexto.
 *
 *	@param program es punto de entrada del programa a ejecutar.
 *	@param parameters es la direccin de memoria que se entregar al programa como parmetro.
 *	@param stack es la menor direccin de memoria RAM del rea reservada para stack.
 *	@param stack_lenght es tamao del stack en palabras (stack_t).
 *	@param wd_config es el tiempo de watchdog (ver io430x54xA para ms detalles con f=32KHz)
 *		Los valores recomendados son:
 *			WDTIS2					1000 ms
 *			WDTIS2+WDTIS0			250 ms
 *			WDTIS2+WDTIS1			16 ms
 *			WDTIS2+WDTIS1+WDTIS0	1.9 ms
 *
 *	@returns Cdigo de error.
 */
extern error_t (*load)(
		void(*program)(void*),
		void* parameters,
		stack_t* stack,
		const size_t stack_length,
		const unsigned short wd_config );

 /**
 *	run
 *	@brief	Ejecuta el despachador, nunca retorna.
 */
extern error_t (*run) ( void );

/**
 *	get_resource_status
 *	@brief	Devuelve el estado de un recurso.
 *
 *	@param id_recurso es el identificador del recurso que se desea investigar
 *	@return el estado del recurso
 */
extern resource_status_t (*get_resource_status)( const resource_t resource );

/**
 * lock
 * @brief Intenta obtener el control sobre un recurso.
 *
 * Los recursos deben ser bloqueados slo por las aplicaciones.
 * Las ISR no deben llamar a esta funcin.
 *
 * Si el recurso est disponible, se marca como bloqueado y se le agrega
 * una referencia al proceso que lo bloquea.
 *
 * Si el recurso est bloqueado por otro proceso, el proceso llamador
 * se agrega a la cola de los que requieren el recurso y se suspende hasta
 * que el recurso est diponible o hata que se alcance el tiempo indicado.
 *
 * @param resource	Identificador del recurso
 * @param timeout	Tiempo mximo de espera expresado en tics.
 *		Si timeout es 0 la espera ser hasta que el recurso est disponible.
 *
 * @return Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_TIMEOUT				Si expir el tiempo
 *		ERROR_ALREADY_LOCKED		Si ese proceso ya tiene bloqueado el recurso
 */
extern error_t (*lock) ( const resource_t resource, const time_t timeout );

/**
 * unlock
 * @brief Libera un recurso previamente bloqueado para que otros procesos puedan utilizarlo.
 * Nunca bloquea.
 * Si exisitan procesos bloqueados a la espera por el recurso, unlock le asigna el recurso al
 * primero de la cola.
 *
 * @param id_recurso es el identificador del recurso a liberar
 *
 * @return	Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_NOT_LOCKED			El recurso no estaba bloqueado
 *		ERROR_INVALID_OWNER		El recurso est bloqueado por otro proceso
 */
extern error_t (*unlock)( const resource_t resource );

/**
 *	set_buffer
 *	@brief La funcin set_buffer define el espacio de transferencia de datos.
 *
 * 	El recurso debe haber sido bloqueado por el proceso llamador.
 *	Si haba datos sin leer en el buffer, reporta el error y define el buffer con
 *	los nuevos parmetros.
 *
 *	@param resource es el identificador del recurso al que se quiere asignar el buffer
 *	@param buffer es el espacio de transferencia
 *	@param buffer_length es la mxima cantidad de bytes que el espacio de transferencia puede alojar.
 *
 *	@return	Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_NOT_LOCKED			Si el recurso no estaba bloqueado
 *		ERROR_INVALID_OWNER		Si recurso est bloqueado por otro proceso
 * 		ERROR_DATA_LOST			Si al establecer el buffer se perdieron datos
 */
extern error_t (*set_buffer)( const resource_t resource, void* buffer, const size_t buffer_length );

/**
 *	reset_buffer
 *	@brief La funcin set_buffer restablece el espacio de transferencia de datos.
 *
 * 	El recurso debe haber sido bloqueado por el proceso llamador.
 *	Si haba datos sin leer en el buffer, reporta el error y restablece el buffer.
 *
 *	@param resource es el identificador del recurso al que se quiere asignar el buffer
 *
 *	@return	Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_NOT_LOCKED			Si el recurso no estaba bloqueado
 *		ERROR_INVALID_OWNER		Si recurso est bloqueado por otro proceso
 * 		ERROR_DATA_LOST			Si al establecer el buffer se perdieron datos
 */
extern error_t (*reset_buffer)( const resource_t resource );

/**
 *	read
 *	@brief Bloquea el proceso hasta que el recurso tenga datos listos para leer o se alcance el tiempo mximo de espera.
 *
 *	@param resource Identificador del recurso por el que se desea esperar
 *	@param data_length es la variable en la que se almacenar el largo de datos
 *	@param timeout es el tiempo mximo de espera en tics
 *
 *	@return	Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_NOT_LOCKED			Si el recurso no estaba bloqueado
 *		ERROR_INVALID_OWNER			Si recurso est bloqueado por otro proceso
 *		ERROR_INVALID_BUFFER		Si no se ha definido o restablecido el buffer de transferencia
 *		ERROR_BUFFER_FULL			Si quedan ms datos para leer
 */
extern error_t (*read)( const resource_t resource, size_t* data_length, const time_t timeout );

/**
 *	write
 *	@brief	La funcin write bloquea el proceso hasta que el recurso referido se encuentre en estado de espera de datos o se alcance tiempo definido.
 *
 *	@param resource			El identificador del recurso por el que se desea esperar
 *	@param buffer			Espacio de transferencia
 *	@param data_length		Largo de datos a enviar
 *	@param writed_length	La cantidad de bytes copiados
 *	@param timeout			Tiempo mximo de espera en tics
 *
 *	Si el largo de datos del write es mayor que el espacio del buffer definido por el proceso
 *	que espera por los datos (el que hizo el set_buffer), se copian tantos bytes como permita el
 *	buffer y se reporta el error a los dos procesos.
 *
 *	@return	Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_BUFFER_FULL			Si el espacio del buffer no permitio transferir todos los datos.
 */
extern error_t (*write) ( const resource_t resource, const void* buffer, const size_t data_length, size_t* writed_length, const time_t timeout );

/**
 *	writei
 *	@brief	La funcin writei transfiere los datos al buffer del recurso si est definido.
 *
 *	@param resource			El identificador del recurso por el que se desea esperar
 *	@param buffer			Espacio de transferencia
 *	@param data_length		Largo de datos a enviar
 *	@param writed_length	La cantidad de bytes copiados
 *
 *	Si el largo de datos del write es mayor que el espacio del buffer definido por el proceso
 *	que espera por los datos (el que hizo el set_buffer), se copian tantos bytes como permita el
 *	buffer y se reporta el error a los dos procesos.
 *
 *	@return	Cdigo de error
 *		ERROR_NO_ERROR				Si el proceso result exitoso
 *		ERROR_INVALID_RESOURCE		Si el recurso no es vlido
 *		ERROR_BUFFER_FULL			Si el espacio del buffer no permitio transferir todos los datos.
 *		ERROR_BUFFER_NOT_READY		Si el recurso no est esperando datos
 */
extern error_t (*writei) ( const resource_t resource, const void* buffer, const size_t data_length,  size_t* writed_length );

/**
 *	tsleep
 *	@brief	Suspende el proceso por al menos por la cantidad de tics indicada.
 *
 *	@param	timeout expresado en tics
 */
extern void (*tsleep)( time_t timeout );

/**
 *	sleep
 *	@brief	Suspende el proceso por al menos por la cantidad de segundos indicada.
 *
 *	@param	segundos
 */
extern void (*sleep)( unsigned long seconds );

/**
 *	stop
 *	@brief	Suspende indefinidamente la ejecucin del proceso.
 */
extern void (*stop)(void);

#endif	/* ifndef __UKERNEL_H__ */
